# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['python_template']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'docutils>=0.18.1,<0.19.0', 'pip>=22.0.4,<23.0.0']

setup_kwargs = {
    'name': 'python-template-ingwersen-erik',
    'version': '0.1.0',
    'description': 'Repositorio template para criacao de projetos em Python.',
    'long_description': '# python-template\n\n<div align="center">\n\n[![Build status](https://github.com/ingwersen-erik/python-template/workflows/build/badge.svg?branch=master&event=push)](https://github.com/ingwersen-erik/python-template/actions?query=workflow%3Abuild)\n[![Python Version](https://img.shields.io/pypi/pyversions/python-template.svg)](https://pypi.org/project/python-template/)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/ingwersen-erik/python-template/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit)\n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/ingwersen-erik/python-template/blob/master/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--versions-e10079.svg)](https://github.com/ingwersen-erik/python-template/releases)\n[![License](https://img.shields.io/github/license/ingwersen-erik/python-template)](https://github.com/ingwersen-erik/python-template/blob/master/LICENSE)\n![Coverage Report](assets/images/coverage.svg)\n\nRepositório template para criação de projetos em Python.\n\n</div>\n\n## Primeiros Passos\n\n### Inicializando o repositório\n\n1. Comece configurando a ferramenta de versionamento de código (VCS).\n   Para inicializar um repositório, utilizando o [GitHub](https://github.com),\n   execute o comando `git`, na linha de comando:\n\n```bash\ncd python-template && git init\n```\n\n**Onde:**\n\n- `cd`: comando é a abreviação de "change directory". Você pode usar esse comando para mudar\n  o diretório da linha de comando.\n- `git init`: inicializa um novo repositório no [GitHub](https://github.com).\n  Vale ressaltar que este comando apenas cria os arquivos necessários para inicializar o repositório, mas não \n  realiza upload dos arquivos para o [GitHub](https://github.com). De certo modo, estamos apenas **"preparando o terreno"** neste momento.\n- `&&`: permite a execução de mais de um comando, de uma só vez. No caso acima, mudamos o diretório do nosso \n  terminal e criamos os arquivos que o [GitHub](https://github.com) precisa.\n\n2. Em seguida, instale o [Poetry](https://tinyurl.com/python-poetry-docs), \n   executando o seguinte comando:\n\nIgnorar esta etapa, caso você já tenha o [Poetry](https://tinyurl.com/python-poetry-docs)\ninstalado na sua maquina.\n\n```bash\nmake poetry-download\n```\n\n> Nunca ouviu falar do [Poetry](https://tinyurl.com/python-poetry-docs)? \n> Clique [aqui](LINK) para uma breve contextualização.\n\n3. Depois de instalar o Poetry, execute o comando abaixo, para instalar as dependências \n   necessárias do pacote:Initialize poetry and install \n   `pre-commit` hooks:\n\n```bash\nmake install\n\n```\n\n**Opcional:** você também pode habilitar uma feature chamada `pre-commit` hooks, executando o seguinte comando:\n\n```bash\nmake pre-commit-install\n```\n\n`pre-commit` hooks são basicamente verificações, que são executadas automaticamente, toda vez que você realiza um \ncommit no seu repositório. Essas verificações variam desde formatar o código fonte, usando ferramentas, como o \n[Black](https://tinyurl.com/black-psf), até execuções de testes unitários. Estes hooks são normalmente utilizados,\nde modo que o commit só é realizado, quando o seu código passa por todas as verificações sem erros. **Caso você não \ntenha muita familiaridade com ferramentas, como [MyPy](https://mypy.readthedocs.io/en/stable/getting_started.html),\n[Pytest](https://pytest.org), [Isort](https://pycqa.github.io/isort/) e [Black](https://tinyurl.com/black-psf), \nrecomendo que não habilite essa opção.**\n\n4. Executando codestyle:\n\nO comando a seguir formatando o seu código fonte, conforme as configurações especificadas dentro do arquivo\n[pyproject.toml](pyproject.toml).\n\n```bash\nmake codestyle\n```\n\n5. Fazendo upload do repositório para o [GitHub](https://github.com):\n\n```bash\ngit add .\ngit commit -m ":tada: Initial commit"\ngit branch -M main\ngit remote add origin https://github.com/ingwersen-erik/python-template.git\ngit push -u origin main\n```\n\nOs comandos acima realizam as seguintes operações:\n\n- `git add .`: Adiciona quaisquer arquivos que você tenha criado/adicionado na pasta do repositório,\n  para serem enviados para o repositório no GitHub.\n- `git commit -m ":tada: Initial commit"`: cria o commit destes arquivos adicionados. Basicamente, um commit\n  especifica as mudanças que pretendemos subir no repositório. A flag `-m` é utilizada para especificar a mensagem\n  do commit e, é obrigatória. Caso você não especifique uma mensagem, o commit nao será feito.\n- `git branch -M main`: cria a branch que iremos utilizar para subir os arquivos. Como estamos criando agora o \n  repositório e, subindo os arquivos do projeto pela primeira vez, estamos criando/utilizando a branch "padrão",\n  que é a branch "main" (alguns projetos preferem utilizar o nome "master", ao invés de "main", muito embora não seja \n  o mais usual).\n- `git remote add origin https://github.com/ingwersen-erik/python-template.git`: esse comando especifica a URL do \n  repositório no Github.\n- `git push -u origin main`: realiza o upload dos arquivos especificados pelo comando `git add .`, utilizando o commit\n  que criamos utilizando o segundo comando (`git commit -m ":tada: Initial commit"`).\n\n### (Opcional) Configurando os bots\n\nAcesse os links abaixo, para instruções de como habilitar o **GitHub Dependabot** e o **Stale bot**.\n\n- Set up [Dependabot](https://docs.github.com/en/github/administering-a-repository/enabling-and-disabling-version-updates#enabling-github-dependabot-version-updates) verifica as dependências do seu projeto e, sugere a \n  atualização de pacotes identificados que possuem algum problema de segurança.\n- Set up [Stale bot](https://github.com/apps/stale) realiza o fechamento automático de GitHub issues estagnados.\n\n### Poetry\n\nWant to know more about Poetry? Check [its documentation](https://python-poetry.org/docs/).\n\n<details>\n<summary>Details about Poetry</summary>\n<p>\n\nPoetry\'s [commands](https://python-poetry.org/docs/cli/#commands) are very intuitive and easy to learn, like:\n\n- `poetry add numpy@latest`\n- `poetry run pytest`\n- `poetry publish --build`\n\netc\n</p>\n</details>\n\n### Building and releasing your package\n\nBuilding a new version of the application contains steps:\n\n- Bump the version of your package `poetry version <version>`.\n  You can pass the new version explicitly, or a rule such as `major`, `minor`, or `patch`.\n  For more details, refer to the [Semantic Versions](https://semver.org/) standard.\n- Make a commit to `GitHub`.\n- Create a `GitHub release`.\n- And... publish 🙂 `poetry publish --build`\n\n## 🎯 What\'s next\n\n### Packages and Articles\n\nHelpful packages and articles:\n\n- [`Typer`](https://github.com/tiangolo/typer) is great for creating CLI applications.\n- [`Rich`](https://github.com/willmcgugan/rich) makes it easy to add beautiful formatting in the terminal.\n- [`Pydantic`](https://github.com/samuelcolvin/pydantic/) – data validation and settings management using Python type hinting.\n- [`Loguru`](https://github.com/Delgan/loguru) makes logging (stupidly) simple.\n- [`tqdm`](https://github.com/tqdm/tqdm) – fast, extensible progress bar for Python and CLI.\n- [`IceCream`](https://github.com/gruns/icecream) is a little library for sweet and creamy debugging.\n- [`orjson`](https://github.com/ijl/orjson) – ultra fast JSON parsing library.\n- [`Returns`](https://github.com/dry-python/returns) makes you function\'s output meaningful, typed, and safe!\n- [`Hydra`](https://github.com/facebookresearch/hydra) is a framework for elegantly configuring complex applications.\n- [`FastAPI`](https://github.com/tiangolo/fastapi) is a type-driven asynchronous web framework.\n\n### Artigos:\n\n- [Open Source Guides](https://opensource.guide/).\n- [A handy guide to financial support for open source](https://github.com/nayafia/lemonade-stand)\n- [GitHub Actions Documentation](https://help.github.com/en/actions).\n- Maybe you would like to add [gitmoji](https://gitmoji.carloscuesta.me/) to commit names. This is really funny. 😄\n\n## 🚀 Features\n\n### Development features\n\n- Supports for `Python 3.9` and higher.\n- [`Poetry`](https://python-poetry.org/) as the dependencies manager. See configuration in [`pyproject.toml`](https://github.com/ingwersen-erik/python-template/blob/master/pyproject.toml) and [`setup.cfg`](https://github.com/ingwersen-erik/python-template/blob/master/setup.cfg).\n- Automatic codestyle with [`black`](https://github.com/psf/black), [`isort`](https://github.com/timothycrosley/isort) and [`pyupgrade`](https://github.com/asottile/pyupgrade).\n- Ready-to-use [`pre-commit`](https://pre-commit.com/) hooks with code-formatting.\n- Type checks with [`mypy`](https://mypy.readthedocs.io); docstring checks with [`darglint`](https://github.com/terrencepreilly/darglint); security checks with [`safety`](https://github.com/pyupio/safety) and [`bandit`](https://github.com/PyCQA/bandit)\n- Testing with [`pytest`](https://docs.pytest.org/en/latest/).\n- Ready-to-use [`.editorconfig`](https://github.com/ingwersen-erik/python-template/blob/master/.editorconfig), [`.dockerignore`](https://github.com/ingwersen-erik/python-template/blob/master/.dockerignore), and [`.gitignore`](https://github.com/ingwersen-erik/python-template/blob/master/.gitignore). You don\'t have to worry about those things.\n\n### Deployment features\n\n- `GitHub` integration: issue and pr templates.\n- `Github Actions` with predefined [build workflow](https://github.com/ingwersen-erik/python-template/blob/master/.github/workflows/build.yml) as the default CI/CD.\n- Everything is already set up for security checks, codestyle checks, code formatting, testing, linting, docker builds, etc with [`Makefile`](https://github.com/ingwersen-erik/python-template/blob/master/Makefile#L89). More details in [makefile-usage](#makefile-usage).\n- [Dockerfile](https://github.com/ingwersen-erik/python-template/blob/master/docker/Dockerfile) for your package.\n- Always up-to-date dependencies with [`@dependabot`](https://dependabot.com/). You will only [enable it](https://docs.github.com/en/github/administering-a-repository/enabling-and-disabling-version-updates#enabling-github-dependabot-version-updates).\n- Automatic drafts of new releases with [`Release Drafter`](https://github.com/marketplace/actions/release-drafter). You may see the list of labels in [`release-drafter.yml`](https://github.com/ingwersen-erik/python-template/blob/master/.github/release-drafter.yml). Works perfectly with [Semantic Versions](https://semver.org/) specification.\n\n### Open source community features\n\n- Ready-to-use [Pull Requests templates](https://github.com/ingwersen-erik/python-template/blob/master/.github/PULL_REQUEST_TEMPLATE.md) and several [Issue templates](https://github.com/ingwersen-erik/python-template/tree/master/.github/ISSUE_TEMPLATE).\n- Files such as: `LICENSE`, `CONTRIBUTING.md`, `CODE_OF_CONDUCT.md`, and `SECURITY.md` are generated automatically.\n- [`Stale bot`](https://github.com/apps/stale) that closes abandoned issues after a period of inactivity. (You will only [need to setup free plan](https://github.com/marketplace/stale)). Configuration is [here](https://github.com/ingwersen-erik/python-template/blob/master/.github/.stale.yml).\n- [Semantic Versions](https://semver.org/) specification with [`Release Drafter`](https://github.com/marketplace/actions/release-drafter).\n\n## Installation\n\n```bash\npip install -U python-template\n```\n\nor install with `Poetry`\n\n```bash\npoetry add python-template\n```\n\n\n### Makefile usage\n\n[`Makefile`](https://github.com/ingwersen-erik/python-template/blob/master/Makefile)\ncontains a lot of functions for faster development.\n\n<details>\n<summary>1. Download and remove Poetry</summary>\n<p>\n\nTo download and install Poetry run:\n\n```bash\nmake poetry-download\n```\n\nTo uninstall\n\n```bash\nmake poetry-remove\n```\n\n</p>\n</details>\n\n<details>\n<summary>2. Install all dependencies and pre-commit hooks</summary>\n<p>\n\nInstall requirements:\n\n```bash\nmake install\n```\n\nPre-commit hooks could be installed after `git init` via:\n\n```bash\nmake pre-commit-install\n```\n\n</p>\n</details>\n\n<details>\n<summary>3. Codestyle</summary>\n<p>\n\nAutomatic formatting uses `pyupgrade`, `isort` and `black`.\n\n```bash\nmake codestyle\n\n# ou use o sinônimo do comando acima\nmake formatting\n```\n\nCodestyle checks only, without rewriting files:\n\n```bash\nmake check-codestyle\n```\n\n> Observação: `check-codestyle` utiliza as seguintes ferramentas: `isort`, `black` e `darglint`.\n\nExecutando o próximo comando, o Poetry atualiza as dependência do seu projeto,\nbuscando a versão mais atualizada de cada biblioteca, caso não existam erros de \ncompatibilidade entre os requerimentos de cada biblioteca.\n\n```bash\nmake update-dev-deps\n```\n\n<details>\n<summary>4. Code security</summary>\n<p>\n\n```bash\nmake check-safety\n```\n\nThis command launches `Poetry` integrity checks as well as identifies security issues with `Safety` and `Bandit`.\n\n```bash\nmake check-safety\n```\n\n</p>\n</details>\n\n</details>\n\n<details>\n<summary>5. Type checks</summary>\n<p>\n\nRun `mypy` static type checker\n\n```bash\nmake mypy\n```\n\n</p>\n</details>\n\n<details>\n<summary>6. Tests with coverage badges</summary>\n<p>\n\nRun `pytest`\n\n```bash\nmake test\n```\n\n</p>\n</details>\n\n<details>\n<summary>7. All linters</summary>\n<p>\n\nOf course there is a command to ~~rule~~ run all linters in one:\n\n```bash\nmake lint\n```\n\nthe same as:\n\n```bash\nmake test && make check-codestyle && make mypy && make check-safety\n```\n\n</p>\n</details>\n\n<details>\n<summary>8. Docker</summary>\n<p>\n\n```bash\nmake docker-build\n```\n\nwhich is equivalent to:\n\n```bash\nmake docker-build VERSION=latest\n```\n\nRemove docker image with\n\n```bash\nmake docker-remove\n```\n\nMore information [about docker](https://github.com/ingwersen-erik/python-template/tree/master/docker).\n\n</p>\n</details>\n\n<details>\n<summary>9. Cleanup</summary>\n<p>\nDelete pycache files\n\n```bash\nmake pycache-remove\n```\n\nRemove package build\n\n```bash\nmake build-remove\n```\n\nDelete .DS_STORE files\n\n```bash\nmake dsstore-remove\n```\n\nRemove .mypycache\n\n```bash\nmake mypycache-remove\n```\n\nOr to remove all above run:\n\n```bash\nmake cleanup\n```\n\n</p>\n</details>\n\n## 📈 Releases\n\nYou can see the list of available releases on the [GitHub Releases](https://github.com/ingwersen-erik/python-template/releases) page.\n\nWe follow [Semantic Versions](https://semver.org/) specification.\n\nWe use [`Release Drafter`](https://github.com/marketplace/actions/release-drafter). As pull requests are merged, a draft release is kept up-to-date listing the changes, ready to publish when you’re ready. With the categories option, you can categorize pull requests in release notes using labels.\n\n### List of labels and corresponding titles\n\n|               **Label**               |  **Title in Releases**  |\n|:-------------------------------------:|:-----------------------:|\n|       `enhancement`, `feature`        |       🚀 Features       |\n| `bug`, `refactoring`, `bugfix`, `fix` | 🔧 Fixes & Refactoring  |\n|       `build`, `ci`, `testing`        | 📦 Build System & CI/CD |\n|              `breaking`               |   💥 Breaking Changes   |\n|            `documentation`            |    📝 Documentation     |\n|            `dependencies`             | ⬆️ Dependencies updates |\n\nYou can update it in [`release-drafter.yml`](https://github.com/ingwersen-erik/python-template/blob/master/.github/release-drafter.yml).\n\nGitHub creates the `bug`, `enhancement`, and `documentation` labels for you. Dependabot creates the `dependencies` label. Create the remaining labels on the Issues tab of your GitHub repository, when you need them.\n\n## 🛡 License\n\n[![License](https://img.shields.io/github/license/ingwersen-erik/python-template)](https://github.com/ingwersen-erik/python-template/blob/master/LICENSE)\n\nThis project is licensed under the terms of the `MIT` license. See [LICENSE](https://github.com/ingwersen-erik/python-template/blob/master/LICENSE) for more details.\n\n## 📃 Citation\n\n```bibtex\n@misc{python-template,\n  author = {ingwersen-erik},\n  title = {Repositorio template para criacao de projetos em Python.},\n  year = {2022},\n  publisher = {GitHub},\n  journal = {GitHub repository},\n  howpublished = {\\url{https://github.com/ingwersen-erik/python-template}}\n}\n```\n\n',
    'author': 'ingwersen-erik',
    'author_email': 'erik.ingwersen@br.ey.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ingwersen-erik/python-template',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
